/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.correio;

import java.io.*;
import java.util.*;

/**
 * Classe que representa uma caixa de mensagens de correio eletronico.
 * Uma caixa pode conter vrias mensagens.
 * Uma caixa pertence a um "titular".
 * <p>A caixa inclui um "cursor" de mensagem. Isto , existe
 * o conceito de "mensagem corrente" e pode-se avanar e recuar
 * na lista de mensagens (mudando assim a mensagem corrente).
 * <p>Algumas operaes podem ser aplicadas  mensagem corrente: excluir, por exemplo.
 * Outras operaes se aplicam  caixa como um todo (salvar, removeCaixaPostal).
 * <p>A caixa postal  salva em disco com a operao salvar.
 * Ao criar uma caixa postal, caso exista uma cpia em disco, a caixa
 *  inicializada com as mensagens que esto no disco.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public class CaixaPostal implements Serializable {
  static final long serialVersionUID = 7121517428757951951L;
  private Vector mensagens;
  private int ndiceMensagemCorrente;
  private String titular;

  /**
   * Construo de uma CaixaPostal annima (sem titular).
   */
  public CaixaPostal() {
    this("");
  }

  /**
   * Construo de uma Caixa Postal
   * Caso haja uma cpia da caixa postal deste titular em disco,
   * as mensagens em disco so carregados na caixa postal sendo criada.
   * No incio, a mensagem corrente ser a primeira mensagem da caixa.
   * @param titular O titular da caixa postal.
   */
  public CaixaPostal(String titular) {
    this.titular = titular;
    ObjectInputStream in = null;
    try {
      try {
        in = new ObjectInputStream(new FileInputStream(getNomeArquivo()));
        mensagens = (Vector)in.readObject();
        in.close();
      } catch( FileNotFoundException e ) {
        // nao achar o arquivo significa que estamos comeando do zero
        mensagens = new Vector();
      }
    } catch(Exception e) {
      System.err.println(e);
      System.exit(1);
    }
    ndiceMensagemCorrente = Math.min(0, mensagens.size()-1);
  }

  protected String getNomeArquivo() {
    return titular + ".correio";
  }
  /**
   * Recupera o titular da caixa postal
   * @return O titular da caixa postal.
   */
  public String getTitular() {
    return titular;
  }

  /**
   * Recupera o nmero de mensagens na caixa postal
   * @return O nmero de mensagens na caixa postal
   */
  public int nmeroDeMensagens() {
    return mensagens.size();
  }

  /**
   * Insira uma nova mensagem no final da caixa postal
   * @param m A Mensagem sendo inserida.
   */
  public void inserir(Mensagem m) {
    mensagens.add(m);
    ndiceMensagemCorrente = Math.max(ndiceMensagemCorrente, 0);
  }

  /**
   * Recupera a mensagem corrente.
   * <p>A caixa inclui um "cursor" de mensagem. Isto , existe
   * o conceito de "mensagem corrente" e pode-se avanar e recuar
   * na lista de mensagens (mudando assim a mensagem corrente).
   * @return A mensagem corrente.
   */
  public Mensagem mensagemCorrente() {
    return ndiceMensagemCorrente >= 0 ? (Mensagem)mensagens.get(ndiceMensagemCorrente) : null;
  }

  /**
   * Avanar o cursor da caixa postal. A mensagem corrente passa a ser a prxima,
   * caso haja. Se a mensagem corrente for a ltima da caixa postal, no h mudana.

   */
  public void avanar() {
    ndiceMensagemCorrente++;
    ndiceMensagemCorrente = Math.min(ndiceMensagemCorrente, mensagens.size()-1);
  }

  /**
   * Recuar o cursor da caixa postal. A mensagem corrente passa a ser a anterior,
   * caso haja. Se a mensagem corrente for a primeira da caixa postal, no h mudana.
   */
  public void recuar() {
    ndiceMensagemCorrente--;
    ndiceMensagemCorrente = Math.max(ndiceMensagemCorrente, 0);
  }

  /**
   * Fornea um iterador para as mensagens da caixa postal.
   * @return O iterador de mensagens.
   */
  public Iterator iterator() {
    return mensagens.iterator();
  }

  /**
   * Excluir a mensagem Corrente da caixa postal.
   * A excluso  apenas lgica. A mensagem est marcada para ser excluda
   * mas s , de fato, excluda ao salvar a caixa postal.
   * @return true, se houve mensagem excluda, false caso contrrio (caixa vazia)
   */
  public boolean excluir() {
    if(ndiceMensagemCorrente >= 0 && ndiceMensagemCorrente < mensagens.size()) {
      mensagemCorrente().excluir();
      ndiceMensagemCorrente = Math.min(ndiceMensagemCorrente, mensagens.size()-1);
      return true;
    } else {
      return false;
    }
  }

  /**
   * Salvar a caixa postal em disco.
   * Neste momento, as mensagens marcadas para excluso so removidas
   * (isto , no so gravadas em disco)
   */
  public void salvar() {
    // primeiro, remover as mensagens excludas
    Iterator it = iterator();
    while(it.hasNext()) {
      Mensagem m = (Mensagem)it.next();
      if(m.isExcluda()) {
        it.remove();
      }
    }
    ObjectOutputStream out = null;
    try {
      try {
        out = new ObjectOutputStream(new FileOutputStream(getNomeArquivo()));
      } catch(FileNotFoundException e) {
        System.err.println("Nao pode criar " + getNomeArquivo());
        System.exit(1);
      }
      out.writeObject(mensagens);
      out.close();
    } catch(IOException e) {
      System.err.println(e);
      System.exit(1);
    }
    ndiceMensagemCorrente = Math.min(ndiceMensagemCorrente, mensagens.size()-1);
  }

  /**
   * Remove uma caixa postal armazenada em disco.
   * @param titular O titular da caixa postal a ser removida.
   */
  public static void removeCaixaPostal(String titular) {
    File f = new File(titular + ".correio");
    try {
      f.delete();
    } catch(Exception e) {}
  }
}
