/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.correio;

import p1.aplic.geral.*;
import java.io.*;

/**
 * Classe abstrata que representa uma mensagem de correio eletronico.
 * Uma mensagem contm um remetente, um assunto uma data de envio e algum contedo.
 * O contedo depende do tipo exato de mensagem (textual, udio).
 * Uma mensagem pode ser exibida (lida) e marcada para excluso.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public abstract class MensagemAbstrata implements Mensagem, Serializable {
  static final long serialVersionUID = 7891762193879048570L;
  protected static final int LIDA = 0x1;
  protected static final int EXCLUDA = 0x2;
  protected static final int NOVA = ~(LIDA | EXCLUDA);

  protected String  remetente;
  protected String  assunto;
  protected Data    dataEnvio;
  protected int     estado;

  protected MensagemAbstrata(String remetente, String assunto) {
    this.remetente = remetente;
    this.assunto = assunto;
    dataEnvio = new Data();
    estado = NOVA;
  }

  /**
   * Recupera o remetente da mensagem
   * @return O remetente da mensagem
   */
  public String getRemetente() {
    return remetente;
  }

  /**
   * Recupera o assunto da mensagem
   * @return O assunto da mensagem
   */
  public String getAssunto() {
    return assunto;
  }

  /**
   * Recupera a data de envio da mensagem
   * @return A data de envio da mensagem
   */
  public Data getDataEnvio() {
    return dataEnvio;
  }

  /**
   * Informa se a mensagem foi lida ou no
   * @return true se a mensagem foi lida, false caso contrrio
   */
  public boolean isLida() {
    return (estado & LIDA) == LIDA;
  }

  /**
   * Informa se a mensagem foi excluda ou no
   * @return true se a mensagem foi excluda, false caso contrrio
   */
  public boolean isExcluda() {
    return (estado & EXCLUDA) == EXCLUDA;
  }

  /**
   * Marcar a mensagem como excluda.
   * A excluso deve ser feita pela coleo que armazena as mensagens.
   * Um exemplo de tal coleo  CaixaPostal.
   */
  public void excluir() {
    estado |= EXCLUDA;
  }

  /**
   * Marcar a mensagem como no excluda.
   */
  public void marcarNoExcluda() {
    estado &= ~EXCLUDA;
  }

  /**
   * Marcar a mensagem como no lida.
   */
  public void marcarNoLida() {
    estado &= ~LIDA;
  }

  /**
   * Testa a igualdade de um objeto com esta mensagem.
   * @param objeto O objeto a comparar com esta mensagem.
   * @return true se o objeto for igual a esta mensagem, false caso contrrio.
   */
  public boolean equals(Object objeto) {
    if(! (objeto instanceof Mensagem)) {
      return false;
    }
    Mensagem outra = (Mensagem)objeto;
    return getRemetente().equals(outra.getRemetente())
           && getAssunto().equals(outra.getAssunto());
  }

  /**
   * Exibir a mensagem. Isso poder imprimir algo na sada
   * ou provocar outras sadas relacionadas com a leitura da mensagem.
   * Aps este mtodo, a mensagem  considerada "lida".
   */
  public abstract void exibir();

  /**
   * Fornea uma representao da mensagem como String
   * @return A representao da mensagem como String.
   */
  public abstract String toString();
}
